# -*- coding:utf-8 -*-
# 文件:  errcode.py

ERR_DATE_FMT = 200
ERR_DATE_FMT_MSG = '“%s”格式错误【%s】，期望 年-月-日！'
E_PUB_NUM = 706
E_PUB_NUM_MSG = '【%s】格式错误，请输入数值！'
E_PUB_NUM_INT = 707
E_PUB_NUM_INT_MSG = '【%s】格式错误，请输入整数！'
E_PUB_NUM_PT2 = 708
E_PUB_NUM_PT2_MSG = '【%s】最多两位小数！'
E_PUB_NEED = 709
E_PUB_NEED_MSG = '请输入【%s】!'
E_PUB_CUSTOMIZED_ERR = 819
E_PUB_CUSTOMIZED_ERR_MSG = '自定义错误提示，通过ppt设置'
